package name.nathanmcrae.numbersstation;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.json.JsonMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Random;
import java.util.ResourceBundle;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.control.Alert;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.KeyCode;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Modality;
import javafx.stage.Stage;

public class MainController implements Initializable {
    private Stage settingsStage;
    private Stage selectStationStage;
    private MainSettings settings;
    private StationSettings selectedStation;

    private StringProperty selectedStationName = new SimpleStringProperty();

    @FXML
    private Label lastRetrievedLabel;

    @FXML
    private TextArea messageTextArea;

    @FXML
    private TextField stationNameField;

    @FXML
    private void handleSettingsButtonPress() {
        System.out.println("Button pressed!");
        lastRetrievedLabel.setText("Button pressed!");

        if (settingsStage == null || !settingsStage.isShowing()) {
            try {
                FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("MainSettingsView.fxml"));
                Parent root = fxmlLoader.load();

                MainSettingsController controller = fxmlLoader.getController();
                if (selectedStation == null) {
                    Alert alert = new Alert(Alert.AlertType.WARNING);
                    alert.setTitle("Invalid Station Selected");
                    alert.setHeaderText(null);
                    alert.setContentText("Invalid station selected. Please choose a station.");
                    alert.showAndWait();
                    return;
                }
                controller.setStationSettings(selectedStation);

                settingsStage = new Stage();
                settingsStage.initModality(Modality.APPLICATION_MODAL);
                settingsStage.setTitle("Numbers Station Settings");
                settingsStage.setScene(new Scene(root));
                settingsStage.show();
                settingsStage.setOnHiding(event -> {
                    updateStationSettings(selectedStation);
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            settingsStage.toFront();
        }
    }

    @FXML
    private void handleSelectStationButtonPress() {
        if (selectStationStage == null || !selectStationStage.isShowing()) {
            try {
                FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("StationSelectionView.fxml"));
                Parent root = fxmlLoader.load();

                StationSelectionController controller = fxmlLoader.getController();
                controller.setSettings(settings);

                selectStationStage = new Stage();
                selectStationStage.initModality(Modality.APPLICATION_MODAL);
                selectStationStage.setUserData("test");
                selectStationStage.setTitle("Numbers Station Selection");
                selectStationStage.setScene(new Scene(root));
                selectStationStage.show();
                selectStationStage.setOnHiding(event -> {
                    String newStationName = (String)selectStationStage.getUserData();
                    StationSettings newSelectedStation = settings.getStations().stream()
                            .filter(station -> station.getName().equals(newStationName))
                            .findFirst()
                            .orElse(null);
                    updateStationSettings(newSelectedStation);
                });
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            selectStationStage.toFront();
        }
    }

    private void loadSettings() throws IOException {
        XmlMapper xmlMapper = new XmlMapper();
        xmlMapper.registerModule(new JavaTimeModule());
        String userHome = System.getProperty("user.home");

        Path filePath = MainSettings.getSettingsFilePath();
        Path directoryPath = filePath.getParent();
        try {
            if (!Files.exists(directoryPath)) {
                Files.createDirectories(directoryPath);
            }

            if (!Files.exists(filePath)) {
                settings = new MainSettings();

                xmlMapper.writeValue(new File(filePath.toString()), settings);
            } else {
                settings = xmlMapper.readValue(new File(filePath.toString()), MainSettings.class);
                for (StationSettings station : settings.getStations()) {
                    if (station.getDigitsPerGroup() == 0) {
                        station.setDigitsPerGroup(4);
                    }
                }
            }
        } catch (IOException e) {
            // Print the contents of filePath
            System.out.println("File contents: " + Files.readString(filePath));
            e.printStackTrace();
        }
    }

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        stationNameField.textProperty().bindBidirectional(selectedStationName);

        try {
            loadSettings();
        } catch (IOException e) {
            e.printStackTrace();
        }

        selectedStationName.set(settings.getSelectedStationName());

        if (selectedStationName.get() == null || selectedStationName.get() == "") {
            selectedStation = settings.getStations().stream().findFirst().orElse(null);
            selectedStationName.set(selectedStation.getName());
        }

        selectedStation = settings.getStations().stream()
            .filter(station -> station.getName().equals(selectedStationName.get()))
            .findFirst()
            .orElse(null);

        if (selectedStation == null) {
            // TODO: create a new station and save it?
        }

        messageTextArea.addEventFilter(KeyEvent.ANY, event -> {
            int cursorPosition = messageTextArea.getCaretPosition();

            String character = event.getCharacter();
            KeyCode code = event.getCode();

            String str = event.getText();

            // Consume the event to block the default behavior
            if (!(code == KeyCode.LEFT ||
                  code == KeyCode.RIGHT ||
                  code == KeyCode.UP ||
                  code == KeyCode.DOWN ||
                  code == KeyCode.HOME ||
                  code == KeyCode.END ||
                  code == KeyCode.PAGE_UP ||
                  code == KeyCode.PAGE_DOWN)) {
                event.consume();
            }

            if (event.getEventType() == KeyEvent.KEY_PRESSED &&
                (code == KeyCode.DIGIT0 ||
                 code == KeyCode.DIGIT1 ||
                 code == KeyCode.DIGIT2 ||
                 code == KeyCode.DIGIT3 ||
                 code == KeyCode.DIGIT4 ||
                 code == KeyCode.DIGIT5 ||
                 code == KeyCode.DIGIT6 ||
                 code == KeyCode.DIGIT7 ||
                 code == KeyCode.DIGIT8 ||
                 code == KeyCode.DIGIT9)) {

                if (cursorPosition < messageTextArea.getLength()) {
                    char currentChar = messageTextArea.getText().charAt(cursorPosition);
                    if (Character.isWhitespace(currentChar)) {
                        messageTextArea.replaceText(cursorPosition + 1, cursorPosition + 2, str);
                    } else {
                        messageTextArea.replaceText(cursorPosition, cursorPosition + 1, str);
                    }
                }
            }
        });
    }

    private void updateStationSettings(StationSettings newStationSettings) {
        if (newStationSettings == null) {
            return;
        }

        selectedStation = newStationSettings;
        selectedStationName.set(newStationSettings.getName());
        settings.setSelectedStationName(selectedStationName.get());

        settings.save();

        // TODO: Load message from file
        // If the message we're overwriting is different from its file, then prompt to confirm

        Random random = new Random();
        StringBuilder messageBuilder = new StringBuilder();

        for (int i = 0; i < selectedStation.getMessageLength(); i++) {
            if (i > 0 && i % selectedStation.getDigitsPerGroup() == 0) {
                messageBuilder.append(" ");
            }
            messageBuilder.append(random.nextInt(10));
        }

        messageTextArea.setText(messageBuilder.toString());
    }
}
